@@ -58,7 +58,6 @@ typedef struct {
   EmpathyTpCall *call;
   EmpathyContact *contact;
   TfChannel *tfchannel;
-  GstBus *bus;
 } EmpathyCallHandlerPriv;
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyCallHandler)
@@ -127,9 +126,6 @@ empathy_call_handler_set_property (GObject *object,
       case PROP_TP_CALL:
         priv->call = g_value_dup_object (value);
         break;
-      case PROP_GST_BUS:
-        priv->bus = g_value_dup_object (value);
-        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -149,9 +145,6 @@ empathy_call_handler_get_property (GObject *object,
       case PROP_TP_CALL:
         g_value_set_object (value, priv->call);
         break;
-      case PROP_GST_BUS:
-        g_value_set_object (value, priv->bus);
-        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -177,12 +170,6 @@ empathy_call_handler_class_init (EmpathyCallHandlerClass *klass)
     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONTACT, param_spec);
 
-  param_spec = g_param_spec_object ("gst-bus",
-    "gst-bus", "The gstreamer bus",
-    GST_TYPE_BUS,
-    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-  g_object_class_install_property (object_class, PROP_GST_BUS, param_spec);
-
   param_spec = g_param_spec_object ("tp-call",
     "tp-call", "The calls channel wrapper",
     EMPATHY_TYPE_TP_CALL,
@@ -232,18 +219,16 @@ empathy_call_handler_new_for_channel (EmpathyTpCall *call)
     "tp-call", call, NULL));
 }
 
-static gboolean
-empathy_call_handler_pipeline_bus_watch (GstBus *bus, GstMessage *message,
-  gpointer user_data)
+void
+empathy_call_handler_bus_message (EmpathyCallHandler *handler,
+  GstBus *bus, GstMessage *message)
 {
-  EmpathyCallHandler *handler = EMPATHY_CALL_HANDLER (user_data);
   EmpathyCallHandlerPriv *priv = GET_PRIV (handler);
 
-  g_assert (priv->tfchannel != NULL);
+  if (priv->tfchannel == NULL)
+    return;
 
   tf_channel_bus_message (priv->tfchannel, message);
-
-  return TRUE;
 }
 
 static void
@@ -251,10 +236,6 @@ empathy_call_handler_tf_channel_session_created_cb (TfChannel *tfchannel,
   FsConference *conference, FsParticipant *participant,
   EmpathyCallHandler *self)
 {
-  EmpathyCallHandlerPriv *priv = GET_PRIV (self);
-
-  gst_bus_add_watch (priv->bus, empathy_call_handler_pipeline_bus_watch, self);
-
   g_signal_emit (G_OBJECT (self), signals[CONFERENCE_ADDED], 0,
     GST_ELEMENT (conference));
 }
@@ -308,6 +289,56 @@ empathy_call_handler_tf_channel_closed_cb (TfChannel *tfchannel,
   g_signal_emit (G_OBJECT (handler), signals[CLOSED], 0);
 }
 
+static GList *
+empathy_call_handler_tf_channel_codec_config_get_defaults (FsCodec *codecs)
+{
+  GList *l = NULL;
+  int i;
+
+  for (i = 0; codecs[i].encoding_name != NULL; i++)
+      l = g_list_append (l, fs_codec_copy (codecs + i));
+
+  return l;
+}
+
+static GList *
+empathy_call_handler_tf_channel_codec_config_cb (TfChannel *channel,
+  guint stream_id, FsMediaType media_type, guint direction, gpointer user_data)
+{
+  FsCodec audio_codecs[] = {
+    { FS_CODEC_ID_ANY, "SPEEX", FS_MEDIA_TYPE_AUDIO, 16000, },
+    { FS_CODEC_ID_ANY, "SPEEX", FS_MEDIA_TYPE_AUDIO, 8000, },
+
+    { FS_CODEC_ID_DISABLE, "DV",     FS_MEDIA_TYPE_AUDIO, },
+    { FS_CODEC_ID_DISABLE, "MPA",    FS_MEDIA_TYPE_AUDIO, },
+    { FS_CODEC_ID_DISABLE, "VORBIS", FS_MEDIA_TYPE_AUDIO, },
+    { FS_CODEC_ID_DISABLE, "MP3",    FS_MEDIA_TYPE_AUDIO, },
+    { 0, NULL, 0,}
+  };
+  FsCodec video_codecs[] = {
+    { FS_CODEC_ID_ANY, "H264",   FS_MEDIA_TYPE_VIDEO, },
+    { FS_CODEC_ID_ANY, "THEORA", FS_MEDIA_TYPE_VIDEO, },
+    { FS_CODEC_ID_ANY, "H263",   FS_MEDIA_TYPE_VIDEO, },
+
+    { FS_CODEC_ID_DISABLE, "DV",   FS_MEDIA_TYPE_VIDEO, },
+    { FS_CODEC_ID_DISABLE, "JPEG", FS_MEDIA_TYPE_VIDEO, },
+    { FS_CODEC_ID_DISABLE, "MPV",  FS_MEDIA_TYPE_VIDEO, },
+    { 0, NULL, 0}
+  };
+
+  switch (media_type)
+    {
+      case FS_MEDIA_TYPE_AUDIO:
+        return empathy_call_handler_tf_channel_codec_config_get_defaults
+          (audio_codecs);
+      case FS_MEDIA_TYPE_VIDEO:
+        return empathy_call_handler_tf_channel_codec_config_get_defaults
+          (video_codecs);
+    }
+
+  return NULL;
+}
+
 static void
 empathy_call_handler_start_tpfs (EmpathyCallHandler *self)
 {
@@ -327,6 +358,8 @@ empathy_call_handler_start_tpfs (EmpathyCallHandler *self)
       G_CALLBACK (empathy_call_handler_tf_channel_stream_created_cb), self);
   g_signal_connect (priv->tfchannel, "closed",
       G_CALLBACK (empathy_call_handler_tf_channel_closed_cb), self);
+  g_signal_connect (priv->tfchannel, "stream-get-codec-config",
+      G_CALLBACK (empathy_call_handler_tf_channel_codec_config_cb), self);
 
   g_object_unref (channel);
 }
@@ -416,14 +449,3 @@ empathy_call_handler_start_call (EmpathyCallHandler *handler)
       empathy_tp_call_accept_incoming_call (priv->call);
     }
 }
-
-void
-empathy_call_handler_set_bus (EmpathyCallHandler *handler, GstBus *bus)
-{
-  EmpathyCallHandlerPriv *priv = GET_PRIV (handler);
-
-  g_assert (priv->bus == NULL);
-
-  priv->bus = g_object_ref (bus);
-}
-
